;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of IndigoGame
;; Copyright 2018 José Agulló García, David Azuar Alonso, Elza Sarrías Alieva
;;
;; IndigoGame is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; IndigoGame is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with IndigoGame.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "cpctelera_functions.h.s"
.include "bullet.h.s"
.include "hero.h.s"
.include "video.h.s"
.include "enemy.h.s" 
.include "maps.h.s"

reload_time = 25
k_max_bullets = 10 ;; cuántos pueden ser como máximo? 
k_bullet_size = bullet_size 


bullet_vector: DefineNBullets b_vector, k_max_bullets 

DefineBullet bullet, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 

num_b:  .db 0x00 
next_b: .dw bullet_vector


aim:
	;;COMPROBAMOS SI SE HA DISPARADO EL ARMA RECIENTEMENTE
	;;NO PERMITIR DISPARAR AL HEROE
	ld a, e_lastShoot(ix)
	or a 
	jp z, startaim
	dec a
	ld e_lastShoot(ix), a
	ret 

	startaim:
	call cpct_scanKeyboard_f_asm
	ld 	 hl, #Key_Space
	call cpct_isKeyPressed_asm
	jp 	 nz, performShoot
	ld 	 hl, #Joy0_Fire1
	call cpct_isKeyPressed_asm
	ret  z
	
	performShoot:
	ld e_lastShoot(ix), #reload_time
	jp shoot


bullet_draw:
	call get_start_video_memory
	ex de, hl 

	ld c, b_x(ix) 
	ld b, b_y(ix) 
	call cpct_getScreenPtr_asm ;; hl contiene el puntero a la posicion en la memoria en video
	
	ex de, hl ;; de contienen el puntero a la pos

	ld a, #0x3c
	ld b, #bullet_h
	ld c, #bullet_w 
	jp cpct_drawSolidBox_asm



check_out_of_bounds:
    ld a, c
    cp #4
	jp z, out_of_bounds
	jp c, out_of_bounds
    cp #76 ;; si la X es mayor que 80 --> fuera del mapa
    jp z, out_of_bounds
    ld a, b
	cp #8
	jp z, out_of_bounds
	jp c, out_of_bounds
    ld a, #0
    or a 
	ret
    out_of_bounds: ld a, #0xff
    or a 
ret


;; comprobar si una coordenada pertenece al mapa o a un obstáculo
;; entrada: registro bc 
;; devuelve 0 en a si no se ha producido colision
;; destruye af, de, bc, hl
check_colision:    
	call get_start_video_memory
	ex de, hl    
	call cpct_getScreenPtr_asm ;; hl contiene el puntero a la posicion en la memoria en video
    ld a, #bg_color
    ld d, a
    ld a, (hl)
    cp d
    jp nz, colision
    ld a, #0
	or a 
    ret
    colision: ld a, #0xff
	or a 
ret



update_b: 
	or a  
	jp p, inc_b 
	dec b 
	ret 
	inc_b:
	inc b 
ret 

update_c:
	or a  
	jp p, inc_c 
	dec c
	ret 
	inc_c: 
	inc c 
ret 

inc_a: 
	inc a 
ret 

dec_a:
	dec a
ret 

bullet_update:
	ld c, b_x(ix) 
	ld a, b_vx(ix) 
	add c 
	ld c, a 

	ld b, b_y(ix) 
	ld a, b_vy(ix) 
	add b 
	ld b, a  

	push ix 
	push bc 
	exx
	pop de 
	call delete_if_bullet_colision 
	exx 
	pop ix 
	or a 
	jp z, not_enemy_colision
		call inc_score_hero
		call restore_need_draw_score
		push ix
		ld l, #1
		ld h, #15
		ld e, #100
		ld d, #0
		ld b, #0
		ld c, #1
		ld a, #4
	    call cpct_akp_SFXPlay_asm
	    pop ix
		jp deactivate

	not_enemy_colision:
	call check_out_of_bounds
	jp nz, deactivate
	push bc 
	call check_colision 
	pop bc
	jp nz, deactivate
	push bc 
	ld a, #bullet_h 
	add b 
	ld b, a
	call check_colision
	pop bc 
	jp nz, deactivate
	push bc 
	ld a, #bullet_w 
	add c 
	ld c, a 
	call check_colision
	pop bc 
	jp nz, deactivate
	
	ld a, b_x(ix) 
	ld b_lx(ix), a 
	ld a, b_y(ix) 
	ld b_ly(ix), a 

	ld b_x(ix), c
	ld b_y(ix), b 
	ret 

	deactivate:
	pop af 
	push ix 
	pop hl 
	exx
	call switch_buffers
	call bullet_clear_current
	call switch_buffers
	exx 
	jp bullet_delete


bullet_clear_last:
	ld b, b_ly(ix) 
	ld c, b_lx(ix)
    jp bullet_clear 

bullet_clear_current:
	ld b, b_y(ix)
	ld c, b_x(ix) 

bullet_clear: 
	ld a, b
	or a  
	ret z 
    call get_start_video_memory
	ex de, hl 
    call cpct_getScreenPtr_asm
    ex de, hl
    ld a, #bg_color 
    ld b, #bullet_h
    ld c, #bullet_w 
    jp cpct_drawSolidBox_asm


shoot:
	;; comprobar que haya espacio en el vector de balas 
	ld a, #k_max_bullets
	ld b, a 
	ld a, (num_b) 
	cp b 
	ret z 

	;; Shoot sound
	ld l, #1
	ld h, #15
	ld e, #40
	ld d, #0
	ld b, #0
	ld c, #1
	ld a, #4
	call cpct_akp_SFXPlay_asm
	;;;;;;;;;;;;;;;;;

    ld ix, #hero_data
	ld iy, #bullet
	ld a, e_x(ix)
	add #04
	ld b_x(iy), a
	ld a, e_y(ix)
	add #08
	ld b_y(iy), a
	

	ld a, e_aimx(ix) 
	ld b_vx(iy), a
	or a  
	jp z, check_aimy
	jp m, left_dir 
	

	ld a, b_x(iy) 
	add a, #04
	ld b_x(iy), a 
	jp check_aimy

	left_dir: 
	ld a, b_x(iy) 
	add #-04
	ld b_x(iy), a 
	;;
	ld a, b_y(iy)
	add a, #-03
	ld b_y(iy), a 
	;;

	check_aimy: 
	ld a, e_aimy(ix) 
	ld b_vy(iy), a 
	or a  
	jp z, set_range
	jp m, up_dir 

	ld a, b_x(iy) 
	add #-01
	ld b_x(iy), a 

	ld a, b_y(iy) 
	add a, #08
	ld b_y(iy), a 
	jp set_range

	up_dir:
	ld a, b_y(iy)
	add a, #-08
	ld b_y(iy), a 

	set_range: 
	
	ld b, b_y(iy) 
	ld c, b_x(iy) 
	call check_colision 
	or a  
	ret nz 
	call bullet_create
	ex de, hl 
	push iy 
	pop hl 
	jp bullet_copy 


;; realizar una acción para todos los elementos de bullet_vector
bullet_do_for_all: 
	ld a, (num_b) 
	or a  
	ret z 
	ld (metodo), hl 
	ld ix, #bullet_vector

	bucle: 
	push af 
	metodo = . +1 
	call bullet_draw 
	pop af 
	ld bc, #k_bullet_size
	add ix, bc 
	dec a 
	jp nz, bucle 
ret 

;; copia los valores de una bala sobre otra 
;; entrada: hl --> origen 
;;			de --> destino 
;; registros destruidos: hl, de, bc 
bullet_copy: 
	ld bc, #k_bullet_size 
	ldir 
ret 


;; añade una bala al vector de balas
;; entrada: -- 
;; devuelve el puntero a la siguiente bala en hl 
;; NO realiza la comprobación del número máximo de las balas 
;; registros destruidos: af, hl, bc  
bullet_create: 
	ld a, (num_b)
	inc a 
	ld (num_b), a 

	ld hl, (next_b)
	ld bc, #k_bullet_size
	add hl, bc 
	ld (next_b), hl 
	or a 
	sbc hl, bc 
ret

;; borra la bala apuntada por hl 
;; entrada: hl --> puntero al enemigo 
;; registros destruidos: af, de, hl, bc 
;; NO realiza la comprobación del número 0 
bullet_delete: 
	ld a, (num_b) 
	dec a 
	ld (num_b), a 
	ex de, hl 
	ld bc, #-k_bullet_size
	ld hl, (next_b)
	add hl, bc 
	ld (next_b), hl 
	or a 
	sbc hl, de 
	ret z 
	add hl, de 
	ld bc, #k_bullet_size
	ldir 
ret 

;; elimina todos los elementos del bullet vector 
;; entrada: --
;; registros destruidos: af, hl 
clear_all_bullets: 
	ld a, #0 
	ld (num_b), a 
	ld hl, #bullet_vector 
	ld (next_b), hl 
ret
	


