;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of IndigoGame
;; Copyright 2018 José Agulló García, David Azuar Alonso, Elza Sarrías Alieva
;;
;; IndigoGame is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; IndigoGame is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with IndigoGame.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "cpctelera_functions.h.s"
.include "assets.h.s"
.include "enemy.h.s"
.include "video.h.s"
.include "hero.h.s"
.include "maps.h.s"
.include "bullet.h.s"

k_enm_size = enemy_data_size

enemy_vector1: DefineNEnemies enemy_vector1, k_max_enm
enemy_vector2: DefineNEnemies enemy_vector2, k_max_enm
enemy_vector3: DefineNEnemies enemy_vector3, k_max_enm
enemy_vector4: DefineNEnemies enemy_vector4, k_max_enm

DefineEnemy enemy_data, 0x8, 0x20, 0x00, 0x00, _skull_sprite_0, enemy_move_move, 0x00

current_vector: .dw #enemy_vector1
num_enm1: .db #0x00
next_enm1: .dw enemy_vector1

num_enm2: .db #0x00
next_enm2: .dw enemy_vector2

num_enm3: .db #0x00
next_enm3: .dw enemy_vector3

num_enm4: .db #0x00
next_enm4: .dw enemy_vector4

current_enemy: .dw 0xffff

mov_table:  .dw #move_NW 
            .dw #move_N 
            .dw #move_NE 
            .dw #move_W 
            .dw #move_none 
            .dw #move_E 
            .dw #move_SW 
            .dw #move_S 
            .dw #move_SE

iterador: .db 0x00
aux_h: .db 0x00
aux_w: .db 0x00 

remove_all_enemies:
    ld a, #0x00
    ld (num_enm4), a
    ld (num_enm3), a 
    ld (num_enm2), a 
    ld (num_enm1), a 
    ld hl, #enemy_vector1
    ld (next_enm1), hl 
    ld hl, #enemy_vector2
    ld (next_enm2), hl 
    ld hl, #enemy_vector3
    ld (next_enm3), hl 
    ld hl, #enemy_vector4
    ld (next_enm4), hl 
ret 

;; entrada: iy --> heroe
check_hero_death: 
    ld d, e_y(iy) 
    ld e, e_x(iy)
    ld b, #hero_h 
    ld c, #hero_w 
    ld hl, #move_none 
    call do_if_cond
    or a ;; 0 --> no hay colision 
ret 

;; de --> coordenadas
check_skull_enemy_colision:
    ld (current_enemy), ix 
    push ix 
    push hl 
    push bc 
    ld d, b 
    ld e, c 
    ld b, #enemy_h 
    ld c, #enemy_w 
    ld hl, #move_none 
    call do_if_cond
    pop bc 
    pop hl 
    pop ix 
    or a ;; 0 --> no hay colision
ret 

;; entrada: de --> yx de bullet 
;; destruye: todo probablemente
delete_if_bullet_colision:  
    ; ld d, b_y(iy) 
    ; ld e, b_x(iy) 
    ld b, #bullet_h 
    ld c, #bullet_w 
    ld hl, #set_dead
    jp do_if_cond

;; hacer algo con un enemigo si colisiona con una entidad (héroe o bala)
;; entrada: de --> yx de entity
;;          bc --> hw de entity 
;;          hl --> metodo 
do_if_cond:  
    exx
    call get_num_enm
    ld (iterador), a 
    exx 
    or a
    ld a, #0x00  
    ret z 

    ; exx
    ; call get_current_vector
    ; exx
    ld ix, (current_vector)

    ld (do_if_metodo), hl 

    do_if_bucle:
    push de 
    push bc 
    call check_entity_enemy_colision
    pop bc 
    pop de 
    jp nz, do_if_cont_bucle 
    do_if_metodo = . + 1
    call move_none 
    jp _fail ;; devuelve 0xff si se ha cumplido la condición
    do_if_cont_bucle:
    ld a, (iterador) 
    push bc 
    ld bc, #k_enm_size
    add ix, bc 
    pop bc 
    dec a 
    ld (iterador), a 
    jp nz, do_if_bucle
    ld a, #0x00 ;; 0x00 si no se ha cumplido 
    ret 


;; entrada: ix --> enemigo 
;;          de --> yx de entity
;;          bc --> hw de entity
;; devuelve 0 en a si hay colision, en hl --> enemigo con el que hay colision
;; destruye: af, hl 
check_entity_enemy_colision: 
    ld a, enm_dead(ix) 
    or a 
    jp nz, _fail ;; si está muerto no hay colision 

    push bc 
    pop hl 

    ld b, enm_y(ix)
    ld c, enm_x(ix) 

    ld a, h
    cp #hero_h 
    jp z, case_hero
    
    ;; entity más pequeño o igual que el enemigo 
    ;; comprobar que de esté dentro del cuadrado marcado por bc 
    cp #enemy_h 
    jp nz, case_bullet

    ;; case_enemy 
    ;; si ix == current_enemy --> se trata del mismo enemigo por tanto no hay colision
    exx 
    push ix 
    pop de 
    ld hl, (current_enemy) 
    or a 
    sbc hl, de 
    jp z, _fail 
    exx 


    case_bullet:
    ld a, #enemy_w  
    ld (aux_w), a 
    ld a, #enemy_h 
    ld (aux_h), a 

    call colision_heurisitic
    jp nz, _fail 

    call check_xy_inside_bc_box
    jp z, entity_colision

    ld a, e 
    add l 
    ld e, a 
    call check_xy_inside_bc_box
    jp z, entity_colision

    ld a, d 
    add h
    ld d, a 
    call check_xy_inside_bc_box   
    jp z, entity_colision

    ld a, e 
    sub l 
    ld e, a 
    call check_xy_inside_bc_box
    jp z, entity_colision
    jp _fail 

    ;; entity más grande que el enemigo 
    ;; comprobar de bc esté dentro del cuadrado marcado por de 
    case_hero:
    push de 
    push bc 
    pop de 
    pop bc 

    ld a, l 
    ld (aux_w), a 
    ld a, h
    ld (aux_h), a 

    call colision_heurisitic
    jp nz, _fail 

    call check_xy_inside_bc_box
    jp z, entity_colision
    ld a, e
    add #enemy_w
    ld e, a 
    call check_xy_inside_bc_box
    jp z, entity_colision
    ld a, d 
    add #enemy_h 
    ld d, a 
    call check_xy_inside_bc_box
    jp z, entity_colision
    ld a, e 
    sub #enemy_w 
    ld e, a 
    call check_xy_inside_bc_box
    jp z, entity_colision

    jp _fail 

    entity_colision:
    ld a, #0x00
    or a
ret 

;; devuelve 0x00 en a si puede haber colision 
;; entrada bc, de 
;; destruye af 
colision_heurisitic:
    call get_difference_b_d 
    exx 
    ld h, a ;; h = diferencia en el eje y 
    ld a, (aux_h) 
    add #enemy_h ;; a = entity_h + enemy_h 
    cp h 
    exx 
    jp c, _fail 
    call get_difference_c_e
    exx 
    ld l, a ;; l = diferencia en el eje x 
    ld a, (aux_w) 
    add #enemy_w ;; a = entity_w + enemy_h 
    cp l 
    exx 
    jp c, _fail 
    ld a, #0x00
    or a 
    ret 
    

;; calcular diferencia absolute de c y e 
get_difference_c_e:
    ld a, c 
    sub e 
    or a 
    ret p 
    neg 
    ret 


;; calcular diferencia absoluta de b y d 
;; destruye a 
get_difference_b_d:
    ld a, b 
    sub d 
    or a 
    ret p 
    neg 
    ret 

;; entrada: bc --> coordenadas del rectangulo
;;          (aux_h) y (aux_l) --> hw del rectangulo
;;          de --> coordenadas a comprobar (d --> y, e --> x)
;; devuelve 0 en a si hay colision 
;; destruye: af
check_xy_inside_bc_box: 
    ld a, c 
    cp e 
    jp c, _x2 
    jp z, _x2 
    jp _fail
    _x2:
    ld a, (aux_w) 
    add c 
    ;dec a 
    cp e 
    jp c, _fail
    _y1: 
    ld a, b 
    cp d 
    jp c, _y2 
    jp z, _y2 
    jp _fail
    _y2:
    ld a, (aux_h) 
    add b 
    ;dec a 
    cp d 
    jp c, _fail
    ld a, #0x00
    or a 
    ret 

;; create and copy 
enemy_add:
    ld ix, #enemy_data 
    call move_method_toggle
    call enemy_create
    or a 
    ret z
    ex de, hl
    ld hl, #enemy_data
    jp enemy_copy

;; entrada: ix --> enemy 
move_method_toggle: 
    ld hl, #enemy_move_move 
    ld d, enm_upd_h(ix) 
    ld e, enm_upd_l(ix) 
    or a 
    sbc hl, de 
    ret nz 
    ld hl, #enemy_pause
    ld enm_upd_h(ix), h 
    ld enm_upd_l(ix), l 
    ret 


;; realizar una acción para todos los elementos de enemy_vector
;; entrada: hl --> metodo a ejecutar 
enemy_do_for_all:
    exx
    call get_num_enm
    exx
    or a
    ret z
    ld (metodo), hl 
    ; call get_current_vector 
    ld ix, (current_vector) 

bucle:
    push af
    metodo = .+1
    call enemy_draw
    pop af
    ld bc, #k_enm_size
    add ix, bc
    dec a
    jp nz, bucle
ret

;; copia los valores de una entidad sobre otra
;; entrada: hl --> origen
;;          de --> destino
;; registros destruidos: hl, de, bc
enemy_copy: 
    ld bc, #k_enm_size
    ldir
ret

;; añade un enemigo al vector de enemigos
;; entrada: -- 
;; devuelve el puntero al siguiente enemigo en hl
;; si no se ha podido crear un enemigo,devuelve 0 en a
;; registros destruidos: af, hl, bc 
enemy_create:
    ld a, #k_max_enm
    ld b, a 
    call get_num_enm
    cp b
    ret z
    inc a 
    call set_num_enm 

    call get_next_enm
    ld bc, #k_enm_size
    add hl, bc
    call set_next_enm
    call get_next_enm
    or a
    sbc hl, bc
ret

set_dead: 
    ld hl, #_explosion_sprite
    ld enm_sp_h(ix), h 
    ld enm_sp_l(ix), l 
    ld hl, #move_none 
    ld enm_upd_h(ix), h 
    ld enm_upd_l(ix), l 
    ld enm_dead(ix), #0x0A
    ret 

enemy_delete_and_clear_buffer:
    exx
    call enemy_clear_current
    call switch_buffers
    call enemy_clear_current
    call switch_buffers
    exx
    jp enemy_delete

;; elimina un enemigo del vector de enemigos
;; entrada: ix --> puntero al enemigo borrado 
;; registros destruidos: af, de, hl, ix, bc
enemy_delete:
    push ix 
    pop hl 
    exx
    call get_num_enm
    or a
    ret z
    ; call get_current_vector
    call get_num_enm
    dec a
    call set_num_enm
    exx
    ex de, hl
    ld ix, (current_vector)
    ld bc, #-k_enm_size
    push de 
    call get_next_enm
    add hl, bc
    call set_next_enm
    pop de 
    or a
    sbc hl, de
    ret z
    add hl, de
    ld bc, #k_enm_size
    ldir 
ret

enemy_draw:
    call get_start_video_memory
    ex de, hl 
    ld c, enm_x(ix)
    ld b, enm_y(ix)
    call cpct_getScreenPtr_asm

    ex de, hl
    ld h, enm_sp_h(ix)
    ld l, enm_sp_l(ix)
    ld b, #enemy_h 
    ld c, #enemy_w 

    call cpct_drawSprite_asm
    ld a, enm_dead(ix) 
    or a 
    ret z 
    dec a 
    ld enm_dead(ix), a 
    or a 
    ret nz 
    call enemy_clear_current
    jp enemy_delete_and_clear_buffer
    
enemy_clear_current:   
    ld c, enm_x(ix) 
    ld b, enm_y(ix) 
    jp enemy_clear

enemy_clear_last:
    ld c, enm_lx(ix) 
    ld b, enm_ly(ix)

enemy_clear: 
    ld a, c 
    or a 
    ret z 
    call get_start_video_memory
    ex de, hl 
    call cpct_getScreenPtr_asm

    ex de, hl
    ld a, #bg_color 
    ld b, #enemy_h 
    ld c, #enemy_w 
    jp cpct_drawSolidBox_asm

enemy_move: 
    ld h, enm_upd_h(ix) 
    ld l, enm_upd_l(ix) 
    jp (hl) 

;; calcula el movimiento y dibuja el enemigo 
enemy_move_move: 
    ld hl, #enemy_pause 
    ld enm_upd_h(ix), h 
    ld enm_upd_l(ix), l 
    ld a, enm_x(ix) 
    ld enm_lx(ix), a 
    ld a, enm_y(ix) 
    ld enm_ly(ix), a 
    jp enemy_ia

enemy_pause:
    ld hl, #enemy_move_move
    ld enm_upd_h(ix), h 
    ld enm_upd_l(ix), l 
ret 

enemy_ia: 
    ld b, enm_y(ix)
    ld a, #enemy_h/2 
    add b 
    ld b, a 
    ld c, enm_x(ix)
    ld a, #enemy_w/2 
    add c 
    ld c, a 
    
    ld iy, #hero_data
    ld hl, #mov_table

    ;; check y
    ld a, e_y(iy)
    add #hero_h/2
    cp b
    jp c, y_lower
    jp nz, y_greater
    ld de, #6
    add hl, de
    y_lower: jp check_x
    y_greater:
    ld de, #12
    add hl, de

    check_x:
    ld a, e_x(iy)
    add #hero_w/2
    cp c
    jp c, x_lower
    jp nz, x_greater
    inc hl
    inc hl
    jp calc_move
    x_lower: jp calc_move
    x_greater:
    ld de, #4 
    add hl, de 

    calc_move: 
    ld a, (hl)
    inc hl 
    ld h, (hl)
    ld l, a

    ;; equivale a call (hl)
    ld bc, #cont 
    push bc 
    jp (hl) 

    cont:
    or a 
    ret nz 
    
    execute_move: 
    ld enm_y(ix), b
    ld enm_x(ix), c
ret



move_NW:
    ld b, enm_y(ix)
    ld c, enm_x(ix)
    ld hl, #_skull_sprite_0
    ld enm_sp_l(ix), l 
    ld enm_sp_h(ix), h 
    ld a, b 
    ld b, #enemy_vy 
    sub b
    ld b, a

    ld a, c 
    ld c, #enemy_vx
    sub c
    ld c, a
    call check_skull_enemy_colision
    jp nz, nw_alt
    call check_colision_upper_left    
    jp nz, nw_alt  
    call check_colision_upper_right 
    jp nz, nw_alt  
    call check_colision_lower_left
    jp nz, nw_alt   
    call check_colision_lower_right 
    jp nz, nw_alt
    ret 
    nw_alt: 
    call move_N
    ret z 
    jp move_W 
    

move_N:
    ld b, enm_y(ix)
    ld c, enm_x(ix)
    ld hl, #_skull_sprite_1
    ld enm_sp_l(ix), l 
    ld enm_sp_h(ix), h 
    ld a, b 
    ld b, #enemy_vy
    sub b
    ld b, a

    call check_skull_enemy_colision
    jp nz, _fail 
    call check_colision_upper_left
    jp nz, _fail     
    call check_colision_lower_left
    jp nz, _fail
    call check_colision_upper_right     
    jp nz, _fail
    call check_colision_lower_right      
    jp nz, _fail
    ld a, #0
    or a 
    ret 


move_NE:
    ld b, enm_y(ix)
    ld c, enm_x(ix)
    ld hl, #_skull_sprite_2
    ld enm_sp_l(ix), l 
    ld enm_sp_h(ix), h 
    ld a, b 
    ld b, #enemy_vy
    sub b
    ld b, a

    ld a, c 
    ld c, #enemy_vx 
    add c
    ld c, a

    call check_skull_enemy_colision
    jp nz, ne_alt
    call check_colision_upper_left    
    jp nz, ne_alt  
    call check_colision_upper_right 
    jp nz, ne_alt  
    call check_colision_lower_left
    jp nz, ne_alt   
    call check_colision_lower_right 
    jp nz, ne_alt
    ret 
    
    ne_alt: 
    call move_N 
    ret z 
    jp move_E 

move_W:
    ld b, enm_y(ix)
    ld c, enm_x(ix)
    ld hl, #_skull_sprite_3
    ld enm_sp_l(ix), l 
    ld enm_sp_h(ix), h 
    ld a, c 
    ld c, #enemy_vx
    sub c
    ld c, a

    call check_skull_enemy_colision
    jp nz, _fail 
    call check_colision_upper_left    
    jp nz, _fail     
    call check_colision_lower_right 
    jp nz, _fail
    call check_colision_lower_left     
    jp nz, _fail
    call check_colision_upper_right     
    jp nz, _fail
    ld a, #0
    or a 
    ret 
     
move_none: ret 

move_E:
    ld b, enm_y(ix)
    ld c, enm_x(ix)
    ld hl, #_skull_sprite_4
    ld enm_sp_l(ix), l 
    ld enm_sp_h(ix), h 
    ld a, c 
    ld c, #enemy_vx
    add c
    ld c, a

    call check_skull_enemy_colision
    jp nz, _fail 
    call check_colision_upper_left    
    jp nz, _fail     
    call check_colision_lower_right 
    jp nz, _fail
    call check_colision_lower_left     
    jp nz, _fail
    call check_colision_upper_right     
    jp nz, _fail
    ld a, #0
    or a
    ret 

move_SW:
    ld b, enm_y(ix)
    ld c, enm_x(ix)
    ld hl, #_skull_sprite_5
    ld enm_sp_l(ix), l 
    ld enm_sp_h(ix), h 
    ld a, b 
    ld b, #enemy_vy 
    add b
    ld b, a

    ld a, c 
    ld c, #enemy_vx 
    sub c
    ld c, a

    call check_skull_enemy_colision
    jp nz, sw_alt
    call check_colision_upper_left    
    jp nz, sw_alt  
    call check_colision_upper_right 
    jp nz, sw_alt  
    call check_colision_lower_left
    jp nz, sw_alt   
    call check_colision_lower_right 
    jp nz, sw_alt
    ret 
    
    sw_alt: 
    call move_S
    ret z 
    jp move_W 

move_S:
    ld b, enm_y(ix)
    ld c, enm_x(ix)
    ld hl, #_skull_sprite_6
    ld enm_sp_l(ix), l 
    ld enm_sp_h(ix), h 
    ld a, b 
    ld b, #enemy_vy 
    add b
    ld b, a

    call check_skull_enemy_colision
    jp nz, _fail 
    call check_colision_upper_left
    jp nz, _fail
    call check_colision_lower_right
    jp nz, _fail
    call check_colision_lower_left 
    jp nz, _fail
    call check_colision_upper_right
    jp nz, _fail
    ld a, #0
    or a
    ret 

move_SE:
    ld b, enm_y(ix)
    ld c, enm_x(ix)
    ld hl, #_skull_sprite_7
    ld enm_sp_l(ix), l 
    ld enm_sp_h(ix), h 
    ld a, b 
    ld b, #enemy_vy 
    add b
    ld b, a

    ld a, c 
    ld c, #enemy_vx 
    add c
    ld c, a

    call check_skull_enemy_colision
    jp nz, se_alt
    call check_colision_upper_left    
    jp nz, se_alt  
    call check_colision_upper_right 
    jp nz, se_alt  
    call check_colision_lower_left
    jp nz, se_alt   
    call check_colision_lower_right 
    jp nz, se_alt
    ret 
    
    se_alt: 
    call move_S
    ret z 
    jp move_E

;; entrada: b --> Y
;;          c --> X
;; destruye: de, af, bc, hl 
check_colision_upper_left:
    push bc 
    call get_start_video_memory
    ex de, hl 
    call cpct_getScreenPtr_asm
    call check_colision
    pop bc 
    ret

;; entrada: b --> Y
;;          c --> X
;;          ix --> puntero a enemigo 
;; destruye: de, af, bc, hl 
check_colision_upper_right: 
    push bc 
    call get_start_video_memory
    ex de, hl 
    ld a, b
    add a, #enemy_h-1
    ld b, a
    call cpct_getScreenPtr_asm
    call check_colision
    pop bc 
    or a
    ret

;; entrada: b --> Y
;;          c --> X
;;          ix --> puntero a enemigo 
;; destruye: de, af, bc, hl 
check_colision_lower_left: 
    push bc 
    call get_start_video_memory
    ex de, hl 
    ld a, c 
    add a, #enemy_w 
    dec a 
    ld c, a 
    call cpct_getScreenPtr_asm
    call check_colision
    pop bc 
    or a
    ret 

;; entrada: b --> Y
;;          c --> X
;;          ix --> puntero a enemigo 
;; destruye: de, af, bc, hl 
check_colision_lower_right:
    push bc 
    call get_start_video_memory
    ex de, hl 
    ld a, b  
    add a, #enemy_h-1 
    ld b, a 
    ld a, c 
    add a, #enemy_w-1
    ld c, a 
    call cpct_getScreenPtr_asm
    call check_colision
    pop bc 
    or a
    ret 

;; comprobar si una coordenada pertenece al mapa o a un obstáculo
;; entrada: registro hl --> posición en memoria
;; devuelve 0 en a si no se ha producido colision
;; destruye af, de
check_colision:    
    ld a, #bg_color
    ld d, a
    ld a, (hl)
    cp d
    jp nz, _fail 
    ld a, #0
    or a 
    ret
    jp _fail 

;; entrada: de --> current map 
;;          hl --> room 
;; devuelve el flag correspondiente 
is_room_x:
    or a ;; set carry = 0
    sbc hl, de ;; hl - de 
ret 

;; checks the current map and loads the correct enemy vector into current_vector 
;; entrada: ---
;; salida: el vector correspondiente en current_vector 
;; destruye: de, iy, hl, af 
get_current_vector:
    call get_current_map_IY
    ld     d, map_this_h(iy)
    ld     e, map_this_l(iy)

    ld hl, (room_1) 
    call is_room_x
    ld hl, #enemy_vector1 
    jp z, load_vector 

    ld hl, (room_2) 
    call is_room_x
    ld hl, #enemy_vector2
    jp z, load_vector
    
    ld hl, (room_3) 
    call is_room_x
    ld hl, #enemy_vector3
    jp z, load_vector

    ld hl, #enemy_vector4 
    load_vector: 
    ld (current_vector), hl 
ret

;; devuelve el num_enm correspondiente en a 
;; destruye de, af 
get_num_enm::
    ld hl, (current_vector) 
    ex de, hl 
    ld hl, #enemy_vector1
    call is_room_x
    ld a, (num_enm1) 
    ret z 


    ld hl, #enemy_vector2 
    call is_room_x 
    ld a, (num_enm2) 
    ret z 

    ld hl, #enemy_vector3
    call is_room_x
    ld a, (num_enm3) 
    ret z 

    ld a, (num_enm4) 
    ret 

;; escribir en a el num_enm correspondiente 
set_num_enm: 
    ld hl, (current_vector) 
    ex de, hl 

    ld hl, #enemy_vector1 
    call is_room_x
    jp nz, num_enm_room2
    ld (num_enm1), a 
    ret 

    num_enm_room2: 
    ld hl, #enemy_vector2
    call is_room_x 
    jp nz, num_enm_room3
    ld (num_enm2), a  
    ret 

    num_enm_room3:
    ld hl, #enemy_vector3
    call is_room_x
    jp nz, num_enm_room4 
    ld (num_enm3), a 
    ret 

    num_enm_room4: 
    ld (num_enm4), a 
    ret 

;; devuelve el next_enm correspondiente en hl 
get_next_enm: 
    ld hl, (current_vector) 
    ex de, hl 

    ld hl, #enemy_vector1
    call is_room_x
    ld hl, (next_enm1) 
    ret z 

    ld hl, #enemy_vector2
    call is_room_x 
    ld hl, (next_enm2)
    ret z 

    ld hl, #enemy_vector3
    call is_room_x
    ld hl, (next_enm3)
    ret z 

    ld hl, (next_enm4) 
    ret 

;; escribe hl en el next_enm correspondiente 
set_next_enm: 
    exx
    ld hl, (current_vector) 
    ex de, hl 

    ld hl, #enemy_vector1
    call is_room_x
    jp nz, next_enm_room2
    exx 
    ld (next_enm1), hl  
    ret 

    next_enm_room2: 
    ld hl, #enemy_vector2
    call is_room_x 
    jp nz, next_enm_room3
    exx 
    ld (next_enm2), hl 
    ret 

    next_enm_room3:
    ld hl, #enemy_vector3
    call is_room_x
    jp nz, next_enm_room4 
    exx 
    ld (next_enm3), hl 
    ret
    
    next_enm_room4:
    exx
    ld (next_enm4), hl 
    ret 


_fail:
    ld a, #0xff 
    or a 
    ret 