;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of IndigoGame
;; Copyright 2018 José Agulló García, David Azuar Alonso, Elza Sarrías Alieva
;;
;; IndigoGame is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; IndigoGame is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with IndigoGame.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "cpctelera_functions.h.s"
.include "assets.h.s"
.include "hero.h.s"
.include "maps.h.s"
.include "video.h.s"

;; DefineEntity  _name, _x, 	 _y, 	   _vx, _vy,  _w, 		 _h, 	  _sp,     _upd			_aimx	_aimy	_lastShoot	_last_x		_last_y
DefineEntity hero_data, hero_ix, hero_iy,  00,  00,  #hero_w,  #hero_h, _hero_0, ent_move_hero, 00,		-hero_aimy,		00,			hero_ix,			hero_iy

hero_moves_x: .db #00
hero_moves_y: .db #00

score_string: .asciz "SCORE:"

numbers_array: .asciz "0" ;; 0
               .asciz "1" ;; 2
               .asciz "2" ;; 4
               .asciz "3" ;; 6
               .asciz "4" ;; 8
               .asciz "5" ;; 10
               .asciz "6" ;; 12
               .asciz "7" ;; 14
               .asciz "8" ;; 16
               .asciz "9" ;; 18


;; restablece los valores por defecto del héroe 
;; destruye: ix, hl 
restore_hero: 
	ld ix, #hero_data
	ld e_x(ix), #hero_ix
	ld e_y(ix), #hero_iy
	ld e_vx(ix), #0
	ld e_vy(ix), #0 
	ld hl, #_hero_0
	ld e_sp_h(ix), h 
	ld e_sp_l(ix), l 
	ld e_aimx(ix), #0
	ld e_aimy(ix), #-hero_aimy
	ld e_lx(ix), #hero_ix 
	ld e_ly(ix), #hero_iy 
	ld e_lastShoot(ix), #0
ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DIBUJAR UNA ENTIDAD
;; REGISTROS DESTRUIDOS: AF, BC, DE, HL
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_draw:
	call check_sprite
	call check_change_map

	ld 	a, #00
	ld 	(hero_moves_x), a
	ld 	(hero_moves_y), a

	call get_start_video_memory	;; Comienzo memoria de video
	ex 	 de, hl
	ld     c, e_x(ix)         	;; C = Entity X
	ld     b, e_y(ix)         	;; B = Entity Y
	call cpct_getScreenPtr_asm

	ex   de, hl   		;; DE = Puntero a memoria
	ld 	 l, e_sp_l(ix)
	ld 	 h, e_sp_h(ix)
	ld 	 b, e_h(ix)   	;; alto
	ld 	 c, e_w(ix)   	;; Ancho

	jp cpct_drawSprite_asm


check_sprite:
	call check_sprite_up 					;; Comprueba si heroe mira hacia arriba
	jr 	 nz, inner_check_sprite_down 		;; -\ Mira hacia arriba
		call check_sprite_right 			;; -- Comprueba si heroe mira hacia la derecha
		jr 	nz, inner_check_sprite_up_left 	;; ----\ Mira hacia a la derecha
			call load_sprite_up_right 		;; ----- Carga el sprite mirando arriba-derecha
			jp check_sprite_end 			;;
											;;
		inner_check_sprite_up_left: 		;; ----/ No mira hacia la derecha
		call check_sprite_left 				;; ----- Comprueba si mira hacia la izquierda
		jr 	nz, inner_load_sprite_up 		;; ------\ Mira hacia la izquierda
			call load_sprite_up_left 		;; ------- Carga el sprite mirando arriba-izquierda
			jp check_sprite_end

inner_load_sprite_up:
	call load_sprite_up 					
	jp check_sprite_end

inner_check_sprite_down:
	call check_sprite_down
	jr 	 nz, inner_check_sprite_left
		call check_sprite_right
		jr 	nz, inner_check_sprite_down_left
			call load_sprite_down_right
			jp check_sprite_end
		
		inner_check_sprite_down_left:
		call check_sprite_left
		jr 	nz, inner_load_sprite_down
			call load_sprite_down_left
			jp check_sprite_end
			
inner_load_sprite_down:
	call load_sprite_down
	jp check_sprite_end

inner_check_sprite_left:
	call check_sprite_left
	jr 	nz, inner_check_sprite_right
		call load_sprite_left
		jp check_sprite_end

inner_check_sprite_right:
	call check_sprite_right
	jr 	nz, check_sprite_end
		call load_sprite_right

check_sprite_end:
	;; ld 	a, #00
	;; ld 	(hero_moves_x), a
	;; ld 	(hero_moves_y), a
	ret   

check_sprite_up:
	ld 	a, (hero_moves_y)
	cp 	#-1
	ret

load_sprite_up:
	ld 	 hl, #_hero_0
	jp load_sprite_HL

check_sprite_right:
	ld 	a, (hero_moves_x)
	cp 	#1
	ret	

load_sprite_right:
	ld 	 hl, #_hero_1
	jp load_sprite_HL

check_sprite_down:
	ld 	a, (hero_moves_y)
	cp 	#1
	ret

load_sprite_down:
	ld 	 hl, #_hero_5
	jp load_sprite_HL

check_sprite_left:
	ld 	a, (hero_moves_x)
	cp 	#-1
	ret

load_sprite_left:
	ld 	 hl, #_hero_4
	jp load_sprite_HL	

load_sprite_up_right:
	ld 	 hl, #_hero_3
	jp load_sprite_HL	

load_sprite_up_left:
	ld 	 hl, #_hero_2
	jp load_sprite_HL	

load_sprite_down_right:
	ld 	 hl, #_hero_7
	jp load_sprite_HL

load_sprite_down_left:
	ld 	 hl, #_hero_6
	jp load_sprite_HL

load_sprite_HL:
   	ld 	e_sp_l(ix), l
   	ld 	e_sp_h(ix), h	
   	ret



ent_clear_last:
    ld b, e_ly(ix)
	ld c, e_lx(ix)
	jp ent_clear

ent_clear_current:
	ld b, e_y(ix) 
	ld c, e_x(ix) 

ent_clear:
	call get_start_video_memory
    ex de, hl 
    call cpct_getScreenPtr_asm

    ex de, hl
    ld a, #bg_color 
    ld b, #hero_h 
    ld c, #hero_w 
    jp cpct_drawSolidBox_asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ACTUALIZAR UNA ENTIDAD
;; REGISTROS DESTRUIDOS:
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_update:
   ld     h, e_up_h(ix)
   ld     l, e_up_l(ix)
   jp    (hl)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MOVER UNA ENTIDAD CON TECLADO
;; REGISTROS DESTRUIDOS:
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_move_hero:
	ld a, e_x(ix) 
	ld e_lx(ix), a 
	ld a, e_y(ix) 
	ld e_ly(ix), a 
	call  cpct_scanKeyboard_f_asm

	ld    hl, #Key_W
	call  cpct_isKeyPressed_asm
	jr    nz, in_w_pulsada
	ld    hl, #Joy0_Up
	call  cpct_isKeyPressed_asm
	jr    z, w_no_pulsada

	in_w_pulsada:
		call dec_hero_moves_y
		jp check_collision_up

w_pulsada:
		;;call inc_score_hero
		ld e_vy(ix), #-hero_vy
		ld e_aimy(ix), #-hero_aimy
		ld e_aimx(ix), #0

w_no_pulsada:
	ld    hl, #Key_D
	call  cpct_isKeyPressed_asm
	jr    nz, in_d_pulsada
	ld    hl, #Joy0_Right
	call  cpct_isKeyPressed_asm
	jr    z, d_no_pulsada

	in_d_pulsada:
		call inc_hero_moves_x
		jp check_collision_right

d_pulsada:
		ld 	 e_vx(ix), #hero_vx
		ld e_aimx(ix), #hero_aimx
		ld e_aimy(ix), #0

d_no_pulsada:
	ld    hl, #Key_S
	call  cpct_isKeyPressed_asm
	jr    nz, in_s_pulsada
	ld    hl, #Joy0_Down
	call  cpct_isKeyPressed_asm
	jr    z, s_no_pulsada

	in_s_pulsada:

		call inc_hero_moves_y
		jp check_collision_down

s_pulsada:
		ld e_vy(ix), #hero_vy
		ld e_aimy(ix), #hero_aimy
		ld e_aimx(ix), #0

s_no_pulsada:
	ld    hl, #Key_A
	call  cpct_isKeyPressed_asm
	jr    nz, in_a_pulsada
	ld    hl, #Joy0_Left
	call  cpct_isKeyPressed_asm
	jr    z, a_no_pulsada

	in_a_pulsada:
	
		call dec_hero_moves_x
		jp check_collision_left

a_pulsada:
		ld e_vx(ix), #-hero_vx
		ld e_aimx(ix), #-hero_aimx
		ld e_aimy(ix), #0

a_no_pulsada:
	call  ent_move
	jp  restore_velocity
	
charge_xy_with_nvy_in_bc:
	ld 	a, e_x(ix)
	ld 	c, a
	ld 	a, e_y(ix)
	add #-hero_vy
	ld 	b, a
	ret

charge_xy_with_vy_in_bc:
	ld 	a, e_x(ix)
	ld 	c, a
	ld 	a, e_y(ix)
	add #hero_vy
	ld 	b, a
	ret

charge_xy_with_nvx_in_bc:
	ld 	a, e_x(ix)
	add #-hero_vx
	ld 	c, a
	ld 	a, e_y(ix)
	ld 	b, a
	ret	

charge_xy_with_vx_in_bc:
	ld 	a, e_x(ix)
	add #hero_vx
	ld 	c, a
	ld 	a, e_y(ix)
	ld 	b, a
	ret	

check_collision_up:
	;; left up corner
	call charge_xy_with_nvy_in_bc
	call check_background_at_CxB
	jp 	 nz, w_no_pulsada

	;; middle up
	call charge_xy_with_nvy_in_bc
	call c_plus_hero_w_div_2
	call check_background_at_CxB
	jp 	 nz, w_no_pulsada

	;; right up corner
	call charge_xy_with_nvy_in_bc
	call c_plus_hero_w
	call check_background_at_CxB
	jp 	 nz, w_no_pulsada
	jp 	  z, w_pulsada

check_collision_down:
	;; left down corner
	call charge_xy_with_vy_in_bc
	call b_plus_hero_h
	call check_background_at_CxB
	jp 	 nz, s_no_pulsada

	;; middle down
	call charge_xy_with_vy_in_bc	
	call b_plus_hero_h
	call c_plus_hero_w_div_2
	call check_background_at_CxB
	jp 	 nz, s_no_pulsada

	;; right down corner
	call charge_xy_with_vy_in_bc
	call c_plus_hero_w
	call b_plus_hero_h
	call check_background_at_CxB
	jp 	 nz, s_no_pulsada
	jp 	  z, s_pulsada

check_collision_left:
	;; left up corner
	call charge_xy_with_nvx_in_bc
	call check_background_at_CxB
	jp 	 nz, a_no_pulsada

	;; middle left
	call charge_xy_with_nvx_in_bc
	call b_plus_hero_h_div_2
	call check_background_at_CxB
	jp 	 nz, a_no_pulsada	

	;; left down corner
	call charge_xy_with_nvx_in_bc
	call b_plus_hero_h
	call check_background_at_CxB
	jp 	 nz, a_no_pulsada
	jp 	  z, a_pulsada

check_collision_right:
	;; right up corner
	call charge_xy_with_vx_in_bc
	call c_plus_hero_w
	call check_background_at_CxB
	jp 	 nz, d_no_pulsada

	;; middle right
	call charge_xy_with_vx_in_bc
	call b_plus_hero_h_div_2
	call c_plus_hero_w
	call check_background_at_CxB
	jp 	 nz, d_no_pulsada

	;; right down corner
	call charge_xy_with_vx_in_bc
	call c_plus_hero_w
	call b_plus_hero_h
	call check_background_at_CxB
	jp 	 nz, d_no_pulsada
	jp 	  z, d_pulsada


;; right
c_plus_hero_w:
	ld 	a, c
	add #hero_w - 1
	ld 	c, a
	ret

c_plus_hero_w_div_2:
	ld 	a, c
	add #hero_w/2 - 1
	ld 	c, a
	ret	

;; down
b_plus_hero_h:
	ld 	a, b
	add #hero_h - 1
	ld 	b, a
	ret

b_plus_hero_h_div_2:
	ld 	a, b
	add #hero_h/2 - 1
	ld 	b, a
	ret

dec_hero_moves_y:
	ld 	 a, (hero_moves_y)
	dec  a
	ld 	(hero_moves_y), a
	ret

inc_hero_moves_y:
	ld 	 a, (hero_moves_y)
	inc  a
	ld 	(hero_moves_y), a
	ret

dec_hero_moves_x:
	ld 	 a, (hero_moves_x)
	dec  a
	ld 	(hero_moves_x), a
	ret

inc_hero_moves_x:
	ld 	 a, (hero_moves_x)
	inc  a
	ld 	(hero_moves_x), a
	ret	


check_background_at_CxB:
	call get_start_video_memory	;;Comienzo memoria de video
	ex 	 de, hl
	call cpct_getScreenPtr_asm
	ld 	 a, (hl)
	cp 	 #bg_color
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MOVER UNA ENTIDAD
;; REGISTROS DESTRUIDOS:
;; ENTRADA: IX -> Puntero a entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ent_move:
	ld    a, e_x(ix)
	add   e_vx(ix)
	ld    e_x(ix), a

	ld    a, e_y(ix) 
	add   e_vy(ix)
	ld    e_y(ix), a
   
	ret

check_change_map:
	call get_current_map_IY

	ld 	a, e_x(ix)
	call check_change_map_right
	call check_change_map_left
	ld 	e_x(ix), a
	ld 	a, e_y(ix)
	call check_change_map_up
	call check_change_map_down
	ld 	e_y(ix), a
	ret

check_change_map_right:
	cp 	#80 - #hero_w
	ret c
	
	call get_right_map_HL
	call update_current_map

	ld 	a, #hero_w
	ret

check_change_map_left:
	cp 	#01
	ret nc

	call get_left_map_HL
	call update_current_map	
	
	ld 	a, #80 - #2*hero_w
	ret

check_change_map_up:
	cp 	#01
	ret nc

	call get_up_map_HL
	call update_current_map

	ld 	a, #200 - #2*hero_h - 48
	ret

check_change_map_down:
	cp 	#200 - hero_h - 40
	ret c

	call get_down_map_HL
	call update_current_map

	ld 	a, #hero_h
	ret

restore_velocity:
	ld e_vx(ix), #00
	ld e_vy(ix), #00
	ret
