;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of IndigoGame
;; Copyright 2018 José Agulló García, David Azuar Alonso, Elza Sarrías Alieva
;;
;; IndigoGame is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; IndigoGame is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with IndigoGame.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

;; Include all CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
.include "cpctelera_functions.h.s"
.include "assets.h.s"
.include "enemy.h.s"
.include "hero.h.s"
.include "maps.h.s"
.include "video.h.s"
.include "bullet.h.s"
.include "menu.h.s"
.include "magic.h.s"
.include "token.h.s"


.globl _song_ingame


;;
;; Start of _DATA area 
;;  SDCC requires at least _DATA and _CODE areas to be declared, but you may use
;;  any one of them for any purpose. Usually, compiler puts _DATA area contents
;;  right after _CODE area contents.
;;
.area _DATA

;;
;; Start of _CODE area
;; 
.area _CODE

;; 
;; Declare all function entry points as global symbols for the compiler.
;; (The linker will know what to do with them)
;; WARNING: Every global symbol declared will be linked, so DO NOT declare 
;; symbols for functions you do not use.
;;

;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;

musictimer: .db #01

isr:
  exx
  ex af, af'
  push af
  push bc
  push de
  push hl
  push iy
  ld a, (musictimer)
  dec a
  ld (musictimer), a
  jr nz, return
    call cpct_akp_musicPlay_asm
    ld a, #6
    ld (musictimer), a
  return:
  pop iy
  pop hl
  pop de
  pop bc
  pop af
  ex af, af'
  exx
  ret

_main::
   ld    sp, #0x8000             ;; change stack pointer for double buffer implementation

   call cpct_disableFirmware_asm
   
   ld    c, #0
   call  cpct_setVideoMode_asm

   ld    hl, #_main_palette
   ld    de, #16
   call  cpct_setPalette_asm

   call  map_initialize

   ;;Initialize music
   ld de, #_song_ingame
   call cpct_akp_musicInit_asm
   ;; set isr
   ld hl, #isr
   call cpct_setInterruptHandler_asm
   ;;Initialize sfx
   ld de, #_song_ingame
   call cpct_akp_SFXInit_asm


menu_loop::
   call  check_option
   call  draw_menu
   call  switch_buffers
   jp    menu_loop

controls_loop::
  call  draw_controls
  call  switch_buffers
  call  menu_check_go_back
  jp    controls_loop

game_over::
  call  draw_game_over
  call  draw_score_go
  call  switch_buffers
  call  menu_check_go_back
  call  menu_check_retry
  jp    game_over

win_loop::
  call  draw_win
  call  switch_buffers
  call  menu_check_go_back
  jp    win_loop

reset_score_main::
  call reset_score
  call reset_magic_timer

game_loop::
    call check_need_draw_score
    jp   z, no_draw_score
      call draw_score
      call dec_need_draw_score

    no_draw_score:
    call check_dec_magic_timer
    call check_need_draw_magic_timer
    jp   z, no_draw_timer
      call draw_magic_timer
      call dec_need_draw_magic_timer
    
    no_draw_timer:
    call check_map_change
    jp z, same_map
  
    call get_current_vector 
    call clear_all_bullets
    call draw_map 
    call switch_buffers
    call draw_map
    call switch_buffers
    jp dif_map

    same_map:
    ld hl, #enemy_clear_last 
    call enemy_do_for_all
    ld hl, #bullet_clear_last 
    call bullet_do_for_all 
    ld ix, #hero_data
    call ent_clear_last

    dif_map:
    ld ix, #hero_data
    call aim 
    
    ld hl, #enemy_clear_current
    call enemy_do_for_all
    ld hl, #bullet_clear_current
    call bullet_do_for_all
    ld hl, #bullet_update 
    call bullet_do_for_all 
    ld hl, #bullet_draw 
    call bullet_do_for_all
    
    ld ix, #enemy_data
    call check_spawn_current_map

    ld hl, #enemy_move
    call enemy_do_for_all
    ld hl, #enemy_draw 
    call enemy_do_for_all
    
    ld iy, #hero_data
    call check_hero_death 
    jp   z, continue_game
    
    hero_death::
    call remove_all_enemies
    call clear_all_bullets
    call restore_hero
    call reset_map 
    jp   game_over

    continue_game::
    ld ix, #hero_data
    call ent_update
    call ent_draw

    call cleartoken
    call drawtoken
    ld ix, #hero_data
    call checkhero
    

    call  cpct_waitVSYNC_asm
    
    call  switch_buffers

    jp    game_loop

  ret