;;-----------------------------LICENSE NOTICE------------------------------------
;; This file is part of IndigoGame
;; Copyright 2018 José Agulló García, David Azuar Alonso, Elza Sarrías Alieva
;;
;; IndigoGame is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; IndigoGame is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with IndigoGame.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------
.include "cpctelera.h.s"
.include "cpctelera_functions.h.s"
.include "assets.h.s"
.include "hero.h.s"
.include "maps.h.s"
.include "video.h.s"
.include "enemy.h.s"
.include "magic.h.s"

;;                       _this   up         right       down        left
room_1: DefineMap map1,  _map1,  #map4,     #map2,      #0xFFFF,    #0xFFFF,  #initial_spawn_timing,  #spawn_timing 
room_2: DefineMap map2,  _map2,  #map3,     #0xFFFF,    #0xFFFF,    #map1,    #initial_spawn_timing,  #spawn_timing
room_3: DefineMap map3,  _map3,  #0xFFFF,   #0xFFFF,    #map2,      #map4,    #initial_spawn_timing,  #spawn_timing
room_4: DefineMap map4,  _map4,  #0xFFFF,   #map3,      #map1,      #0xFFFF,  #initial_spawn_timing,  #spawn_timing

k_current_map: .dw #map1
last_map: .dw #map1 

score_hero_001: .db #0
score_hero_010: .db #0
score_hero_100: .db #0


need_draw_score: .db #2

reset_map: 
   ld hl, #map1
   ld (k_current_map), hl 
   ld hl, #0x0000 
   ld (last_map), hl
   ret 

check_map_change: 
	call get_current_map_IY 
	ld d, map_this_h(iy) 
	ld e, map_this_l(iy) ;; en de --> mapa actual 
	ld hl, (last_map) ;; en hl --> ultima mapa guardado 
	or a  
	sbc hl, de 
      jp nz, dif_map 
      ld a, #0x00 
      cp #0 
	ret ;; el mismo mapa 
	dif_map:
      ex de, hl 
	ld (last_map), hl 
	ld a, #0xff 
	ret ;; diferente mapa

map_initialize:
   ld    hl, #_map_tileset_00
   ld    c, #160/8
   ld    b, #200/8
   ld    de, #map_w_tiles
   jp  cpct_etm_setDrawTilemap4x8_ag_asm

draw_map:
   call  get_current_map_IY
   ld     d, map_this_h(iy)
   ld     e, map_this_l(iy)

   call  get_start_video_memory
   call  cpct_etm_drawTilemap4x8_ag_asm
   call  draw_score
   jp    draw_magic_timer

update_current_map:
   ld    (k_current_map), hl
   ;; ld    ix, (k_current_map) 
   ret

check_can_change_map:
   ld    a, #0xFF
   sub   h
   add   a, #0xFF
   sub   l
   cp    #0
   ret

get_current_map_IY:
   ld iy, (k_current_map)
   ret

get_right_map_HL:
   ld    h, map_right_h(iy)
   ld    l, map_right_l(iy)
   ret

get_left_map_HL:
   ld    h, map_left_h(iy)
   ld    l, map_left_l(iy)
   ret

get_down_map_HL:
   ld    h, map_down_h(iy)
   ld    l, map_down_l(iy)
   ret

get_up_map_HL:
   ld    h, map_up_h(iy)
   ld    l, map_up_l(iy)
   ret  

check_spawn_current_map:
   call get_num_enm        ;; a = num_enm(current_map)
   call get_current_map_IY
   cp   #k_max_enm / 2
   jp   nc, check_spawn_timing
      ld    a, map_itiming(iy)
      cp    #0
      jr    z, inner_spawn_enemy
         dec a
         ld  map_itiming(iy), a
         ret

check_spawn_timing:
      ld    a, map_timing(iy)
      cp    #0
      jr    z, inner_spawn_enemy
         dec a
         ld  map_timing(iy), a
         ret

inner_spawn_enemy:
   call  random_spawn_coordinates
   ld    map_itiming(iy), #initial_spawn_timing
   ld    map_timing(iy),  #spawn_timing
   jp  enemy_add

;; entrada: ix <- enemy_data
random_spawn_coordinates:
   call get_random_0_3

;; rand0
   cp   #0
   jr    nz, rand1
      ld enm_x(ix), #4
      ld enm_y(ix), #16

rand1:
   cp   #1
   jr    nz, rand2
      ld enm_x(ix), #80 - 10
      ld enm_y(ix), #16
rand2:
   cp   #2
   jr    nz, rand3
      ld enm_x(ix), #4
      ld enm_y(ix), #200 - 60
rand3:
   cp   #3
   ret nz
      ld enm_x(ix), #80 - 10
      ld enm_y(ix), #200 - 60
   ret

get_random_0_3:
   call cpct_getRandom_mxor_u8_asm
   ld    a, l
   ld    b, #03
   and   b
   ret

draw_score:
   ld    l, #0                   ;; D = Background PEN (0)
   ld    h, #1                   ;; E = Foreground PEN (3)
   call cpct_setDrawCharM0_asm   ;; Set draw char colours

   call get_start_video_memory
   ld   bc, #80 * 22 + 4
   add  hl, bc 

   ld   iy, #score_string      ;; IY = Pointer to the string 
   call cpct_drawStringM0_asm  ;; Draw the string

   call print_100
   call print_010
   jp print_001

print_100:
   ld a, (score_hero_100)

   call a_to_char_in_iy

   call get_start_video_memory
   ld   bc, #80 * 22 + 28
   add  hl, bc 
   jp cpct_drawStringM0_asm  ;; Draw the string

print_010:
   ld a, (score_hero_010)

   call a_to_char_in_iy

   call get_start_video_memory
   ld   bc, #80 * 22 + 32
   add  hl, bc 
   jp cpct_drawStringM0_asm  ;; Draw the string

print_001:
   ld a, (score_hero_001)

   call a_to_char_in_iy

   call get_start_video_memory
   ld   bc, #80 * 22 + 36
   add  hl, bc 
   jp cpct_drawStringM0_asm  ;; Draw the string

draw_score_go:
   ld    l, #6                   ;; D = Background PEN (0)
   ld    h, #3                   ;; E = Foreground PEN (3)
   call cpct_setDrawCharM0_asm   ;; Set draw char colours

   call get_start_video_memory
   ld   bc, #80 * 19 + 20
   add  hl, bc 

   ld   iy, #score_string      ;; IY = Pointer to the string 
   call cpct_drawStringM0_asm  ;; Draw the string

   call print_100_go
   call print_010_go
   jp print_001_go

print_100_go:
   ld a, (score_hero_100)

   call a_to_char_in_iy

   call get_start_video_memory
   ld   bc, #80 * 19 + 20 + 24 + 4*0
   add  hl, bc 
   jp cpct_drawStringM0_asm  ;; Draw the string

print_010_go:
   ld a, (score_hero_010)

   call a_to_char_in_iy

   call get_start_video_memory
   ld   bc, #80 * 19 + 20 + 24 + 4*1
   add  hl, bc 
   jp cpct_drawStringM0_asm  ;; Draw the string

print_001_go:
   ld a, (score_hero_001)

   call a_to_char_in_iy

   call get_start_video_memory
   ld   bc, #80 * 19 + 20 + 24 + 4*2
   add  hl, bc 
   jp cpct_drawStringM0_asm  ;; Draw the string

a_to_char_in_iy:
   ld hl, #numbers_array
   ld bc, #2
   ld  d, #1
   jp n_1

n_1_s:
   add hl, bc
   sub d
n_1:
   cp  d
   jp  nc, n_1_s

   push hl
   pop  iy

   ret

inc_score_hero:
   ld a, (score_hero_001)
   cp #9
   jp z, check_010
   inc a
   ld (score_hero_001), a
   ret

check_010:
   ld a, #0
   ld (score_hero_001), a

   ld a, (score_hero_010)
   cp #9
   jp z, check_100
   inc a
   ld (score_hero_010), a
   ret

check_100:
   ld a, #0
   ld (score_hero_010), a

   ld a, (score_hero_100)
   cp #9
   jp nz, no_win
      call remove_all_enemies
      call clear_all_bullets
      call restore_hero
      call reset_map 
      jp   win_loop

no_win:
   inc a
   ld (score_hero_100), a

   ret

reset_score:
   ld    a, #0
   ld    (score_hero_001), a
   ld    (score_hero_010), a
   ld    (score_hero_100), a
   ret

check_need_draw_score:
   ld  a, (need_draw_score)
   cp  #0
   ret

dec_need_draw_score:
   ld    a, (need_draw_score)
   dec   a
   ld    (need_draw_score), a
   ret

restore_need_draw_score:
   ld    a, #2
   ld    (need_draw_score), a
   ret